/***************************************************************************
 *   Copyright (C) 2024~2024 by liulinsong                                 *
 *   liulinsong@kylinos.cn                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.              *
 ***************************************************************************/

#ifndef REQUESTMERGER_H
#define REQUESTMERGER_H

#include "fcitx-utils/utils.h"
#include "fcitx/instance.h"

enum RequestMergerState { Deactivation = -1, Idle = 0, Activation = 1 };

typedef void (*ExecuteCallback)(void *owner);
typedef boolean (*ShouldActivateCallback)(void *owner);
typedef boolean (*ShouldDeactivateCallback)(void *owner);

typedef struct _RequestMerger {
    void *owner_;
    FcitxInstance *instance_;

    long int period_;

    ExecuteCallback activationExecuteCallback_;
    ExecuteCallback deactivationExecuteCallback_;

    ShouldActivateCallback shouldActivateCallback_;
    ShouldDeactivateCallback shouldDeactivateCallback_;

    enum RequestMergerState currentState_;
    uint64_t timerId_;
} RequestMerger;

RequestMerger *RequestMerger_new(void *owner, FcitxInstance *instance,
                                 long int period);
void RequestMerger_delete(RequestMerger *self);

void RequestMerger_init(RequestMerger *self,
                        ExecuteCallback activationExecuteCallback,
                        ExecuteCallback deactivationExecuteCallback,
                        ShouldActivateCallback shouldAcitvateCallback,
                        ShouldDeactivateCallback shouldDeactivateCallback);

void RequestMerger_activate(RequestMerger *self);
void RequestMerger_deactivate(RequestMerger *self);

void RequestMerger_enterIdleState(RequestMerger *self);

#endif // REQUESTMERGER_H
